
from Products.ATContentTypes.config import HAS_LINGUA_PLONE
from Products.blob.zopefile import OFSBlobFile, FileBlob

if HAS_LINGUA_PLONE:
    from Products.LinguaPlone.public import *
else:
    from Products.Archetypes.public import *

from Products.Archetypes.interfaces.base import IBaseUnit
from OFS.Image import File, Pdata
from ZPublisher.HTTPRequest import FileUpload
from types import FileType
from Products.Archetypes.utils import shasattr
from Products.Archetypes.exceptions import FileFieldException
from Products.CMFCore.utils import getToolByName
from OFS.content_types import guess_content_type

class BlobFileField(FileField):

    __implements__ = FileField.__implements__

    content_class = OFSBlobFile
    _properties = FileField._properties.copy()
    _properties.update({
        'content_class' : OFSBlobFile,
        })


    def _process_input(self, value, file=None, default=None, mimetype=None,
                       instance=None, filename='', **kwargs):
        if file is None:
            file = self._make_file(self.getName(), title='',
                                   file='', instance=instance)
        if IBaseUnit.isImplementedBy(value):
            mimetype = value.getContentType() or mimetype
            filename = value.getFilename() or filename
            value = value.getRaw()
        elif isinstance(value, self.content_class):
            filename = getattr(value, 'filename', value.getId())
            mimetype = getattr(value, 'content_type', mimetype)
            return value, mimetype, filename
        elif isinstance(value, File):
            # In case someone changes the 'content_class'
            filename = getattr(value, 'filename', value.getId())
            mimetype = getattr(value, 'content_type', mimetype)
            value = value.data
        elif isinstance(value, FileUpload) or shasattr(value, 'filename'):
            filename = value.filename
        elif isinstance(value, FileType) or shasattr(value, 'name'):
            # In this case, give preference to a filename that has
            # been detected before. Usually happens when coming from PUT().
            filename = filename or value.name
            # Should we really special case here?
            if filename == '<fdopen>':
                filename = ''
        elif isinstance(value, basestring):
            # Let it go, mimetypes_registry will be used below if available
            # if mimetype is None:
            #     mimetype, enc = guess_content_type(filename, value, mimetype)
            pass
        elif (isinstance(value, Pdata) or (shasattr(value, 'read') and
                                           shasattr(value, 'seek'))):
            # Can't get filename from those.
            pass
        elif value is None:
            # Special case for setDefault
            value = ''
        else:
            klass = getattr(value, '__class__', None)
            raise FileFieldException('Value is not File or String (%s - %s)' %
                                     (type(value), klass))
        filename = filename[max(filename.rfind('/'),
                                filename.rfind('\\'),
                                filename.rfind(':'),
                                )+1:]
        if filename:
            # LeoRochael: this is a shameful hack to mangle the filename
            # into the fileblog uid so that external systems can get
            # proper filesystem extensions into the filesystem file.
            file.data = FileBlob(file.data.uid.split("_", 1)[0] +
                                 '_' + filename)
        file.manage_upload(value)
        if mimetype is None or mimetype == 'text/x-unknown-content-type':
            body = file.getIterator().read(8096)
            mtr = getToolByName(instance, 'mimetypes_registry', None)
            if mtr is not None:
                kw = {'mimetype':None,
                      'filename':filename}
                d, f, mimetype = mtr(body[:8096], **kw)
            else:
                mimetype = getattr(file, 'content_type', None)
                if mimetype is None:
                    mimetype, enc = guess_content_type(filename, body, mimetype)
        # mimetype, if coming from request can be like:
        # text/plain; charset='utf-8'
        mimetype = str(mimetype).split(';')[0].strip()
        setattr(file, 'content_type', mimetype)
        setattr(file, 'filename', filename)
        return file, mimetype, filename
