
import os

from Products.ATContentTypes.config import HAS_LINGUA_PLONE

if HAS_LINGUA_PLONE:
    from Products.LinguaPlone.public import *
else:
    from Products.Archetypes.public import *

from Products.validation import V_REQUIRED

from Products.ATContentTypes.content.file import ATFile, ATFileSchema
from Products.CMFCore.utils import getToolByName

from Products.CMFCore.permissions import View, ModifyPortalContent
from AccessControl import ClassSecurityInfo
from Products.Archetypes.utils import shasattr, contentDispositionHeader

from Products.BlobFile.field import BlobFileField
from Products.BlobFile.config import PROJECT_NAME, TOOL_NAME
from Products.BlobFile.marshall import BlobMarshaller
from ComputedAttribute import ComputedAttribute
from Acquisition import aq_base

BlobFileBaseSchema = ATFileSchema.copy()
BlobFileBaseSchema['file'] = BlobFileField('file',
              required=True,
              primary=True,
              languageIndependent=True,
              storage = AnnotationStorage(migrate=True),
              validators = (('isNonEmptyFile', V_REQUIRED),
                             ('checkFileMaxSize', V_REQUIRED)),
              widget = FileWidget(
                        #description = "Select the file to be added by clicking the 'Browse' button.",
                        #description_msgid = "help_file",
                        description = "",
                        label= "File",
                        label_msgid = "label_file",
                        i18n_domain = "plone",
                        show_content_type = False,))

BlobFileSchema = BlobFileBaseSchema
BlobFileSchema.registerLayer('marshall',  BlobMarshaller())

class BlobFile(ATFile):

    schema = BlobFileSchema
    meta_type = portal_type = archetype_name = 'BlobFile'
    _blob_dir = ''

    __implements__ = ATFile.__implements__

    security = ClassSecurityInfo()

    def _setBlobDir(self):
        if not self._blob_dir:
            btool = getToolByName(self, TOOL_NAME)
            blob_dir = btool.getBlobDir()
            self._blob_dir = blob_dir
        os.environ['BLOB_DIR'] = self._blob_dir

    security.declareProtected(ModifyPortalContent, 'setFile')
    def setFile(self, value, **kwargs):
        self._setBlobDir()
        ATFile.setFile(self, value, **kwargs)

    security.declareProtected(View, 'getFile')
    def getFile(self):
        self._setBlobDir()
        return ATFile.getFile(self)

    security.declareProtected(View, 'download')
    def download(self, REQUEST=None, RESPONSE=None):
        """Download the file (use default index_html)
        """
        self._setBlobDir()
        if REQUEST is None:
            REQUEST = self.REQUEST
        if RESPONSE is None:
            RESPONSE = REQUEST.RESPONSE
        RESPONSE.setHeader('Content-Disposition',
                contentDispositionHeader('attachment',
                                         charset=self.getCharset(),
                                         filename="%s" % self.getId()))
        return self.manage_FTPget(REQUEST, RESPONSE)
        #field = self.getPrimaryField()
        #return field.download(self, REQUEST, RESPONSE)

    security.declareProtected(View, 'index_html')
    def index_html(self, REQUEST=None, RESPONSE=None):
        """Make it directly viewable when entering the objects URL
        """
        return self.download(REQUEST, RESPONSE)

    security.declareProtected(View, 'manage_FTPget')
    def manage_FTPget(self, REQUEST=None, RESPONSE=None):
        "Get the raw content for this object (also used for the WebDAV SRC)"

        if REQUEST is None:
            REQUEST = self.REQUEST

        if RESPONSE is None:
            RESPONSE = REQUEST.RESPONSE

        if not self.Schema().hasLayer('marshall'):
            RESPONSE.setStatus(501) # Not implemented
            return RESPONSE

        # Should always call this.
        self.dav__init(REQUEST, RESPONSE)

        marshaller = self.Schema().getLayerImpl('marshall')
        ddata = marshaller.marshall(self, REQUEST=REQUEST, RESPONSE=RESPONSE)
        if shasattr(self, 'marshall_hook') \
           and self.marshall_hook:
            ddata = self.marshall_hook(ddata)

        content_type, length, data = ddata

        RESPONSE.setHeader('Content-Type', content_type)
        RESPONSE.setHeader('Content-Length', length)

        if type(data) is type(''): return data

        if hasattr(data, 'data'):
            while data is not None:
                RESPONSE.write(data.data)
                data=data.next
        else:
            for block in data:
                RESPONSE.write(block)

    security.declareProtected(View, 'get_data')
    def get_data(self):
        """CMF compatibility method
        """
        self._setBlobDir()
        data = aq_base(self.getPrimaryField().getAccessor(self)())
        return str(data)

    data = ComputedAttribute(get_data, 1)

registerType(BlobFile, PROJECT_NAME)

