
from Products.ATContentTypes.config import HAS_LINGUA_PLONE

if HAS_LINGUA_PLONE:
    from Products.LinguaPlone.public import *
else:
    from Products.Archetypes.public import *

from AccessControl import ClassSecurityInfo
from Products.Archetypes.utils import shasattr
from Globals import InitializeClass
from OFS.Image import File
from Products.Archetypes.interfaces.base import IBaseUnit
from OFS.content_types import guess_content_type
from Products.blob.zopefile import OFSBlobFile


class BlobMarshaller(PrimaryFieldMarshaller):

    security = ClassSecurityInfo()
    security.declareObjectPrivate()
    security.setDefaultAccess('deny')


    def marshall(self, instance, **kwargs):
        p = instance.getPrimaryField()
        if not p:
            raise TypeError, 'Primary Field could not be found.'
        data = p and instance[p.getName()] or ''
        content_type = length = None
        # Gather/Guess content type
        if IBaseUnit.isImplementedBy(data):
            content_type = data.getContentType()
            length = data.get_size()
            data   = data.getRaw()
        elif isinstance(data, OFSBlobFile):
            content_type = data.getContentType()
            length = data.get_size()
            data = data.getIterator()
        elif isinstance(data, File):
            content_type = data.content_type
            length = data.get_size()
            data = data.data
        else:
            if hasattr(p, 'getContentType'):
                content_type = p.getContentType(instance) or 'text/plain'
            else:
                content_type = data and guess_content_type(data) or 'text/plain'
            # XXX FIXME
            # DM 2004-12-01: "FileField"s represent a major field class
            #  that does not use "IBaseUnit" yet.
            #  Ensure, the used "File" objects get the correct length.
            if hasattr(p, 'get_size'):
                length = p.get_size(instance)
            else:
                # DM: this almost surely is stupid!
                length = len(data)

            length = len(data)
            # ObjectField without IBaseUnit?
            if shasattr(data, 'data'):
                data = data.data
            else:
                data = str(data)
                # DM 2004-12-01: recompute 'length' as we now know it definitely
                length = len(data)

        return (content_type, length, data)

InitializeClass(BlobMarshaller)
