
import os
from sets import Set

from OFS.SimpleItem import SimpleItem
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore.permissions import ManagePortal, View
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.blob.file import filename_from_blob

from Products.BlobFile.config import TOOL_NAME
from Products.BlobFile.field import BlobFileField

default_dir = os.path.join(CLIENT_HOME, 'filedata')
_www = os.path.join(os.path.dirname(__file__), 'www')

class BlobTool(UniqueObject, SimpleItem):
    """ BlobFile support tool
    """

    security = ClassSecurityInfo()

    id = TOOL_NAME
    meta_type = 'Blob Tool'
    title = 'BlobFile support tool'
    plone_tool = True

    _blob_dir = default_dir

    _monitored_types = ()

    manage_options = (({'label': 'Configure',
                        'action': 'manage_configureForm'
                        },
                        {'label': 'Orphans',
                        'action': 'manage_reportOrphans'
                        },
                        )
                        + SimpleItem.manage_options
                        )


    __implements__ = (SimpleItem.__implements__,)

    security.declareProtected(ManagePortal, 'manage_configureForm')
    manage_configureForm = PageTemplateFile('configure', _www)
    manage_reportOrphans = PageTemplateFile('orphans', _www)

    security.declareProtected(View, 'getBlobDir')
    def getBlobDir(self):
        return self._blob_dir

    security.declareProtected(ManagePortal, 'manage_configure')
    def manage_configure(self, blob_dir='', REQUEST=None):
        """ edit the tool configuration
        """
        request = REQUEST
        if request is not None:
            response = request.RESPONSE
        url_pattern = '%s/manage_configureForm?manage_tabs_message=%s'
        if blob_dir and os.path.exists(blob_dir):
            self._blob_dir = blob_dir
            url = url_pattern % (self.absolute_url(),
                                 'Configuration+changed.')
        else:
            if request is None:
                raise ValueError, '%s does not exist.' % blob_dir
            url = url_pattern % (self.absolute_url(),
                'directory+does+not+exist.')
        if request is not None:
            response.redirect(url)


    security.declareProtected(ManagePortal, 'getMonitoredTypes')
    def getMonitoredTypes(self):
        return self._monitored_types

    security.declareProtected(ManagePortal, 'manage_setMonitoredTypes')
    def manage_setMonitoredTypes(self, monitored_types, REQUEST=None):
        """ edit the types to report orphans
        """
        request = REQUEST
        if request is not None:
            response = request.RESPONSE
        url_pattern = '%s/manage_reportOrphans?manage_tabs_message=%s'
        if isinstance(monitored_types, basestring):
            monitored_types = [line.strip() for line in monitored_types.split("\n") if line.strip()]
        valid_types = getToolByName(self, 'portal_types').objectIds()
        for typename in monitored_types:
            assert typename in valid_types, "no '%s' in portal_types" % typename
        self._monitored_types = tuple(monitored_types)
        url = url_pattern % (self.absolute_url(),
                            'Monitored+types+changed.')

        if request is not None:
            response.redirect(url)

    security.declareProtected(ManagePortal, 'listOrphans')
    def listOrphans(self, monitored_types=None, REQUEST=None):
        if monitored_types is None:
            monitored_types = self.getMonitoredTypes()
            if not monitored_types:
                return []

        cat = getToolByName(self, 'portal_catalog')
        known_files = Set([self.getFilenameForContent(rec.getObject())
                           for rec in cat(portal_type=monitored_types)])
        # walk blob_dir to hunt for orphans
        orphans = []
        for dirpath, dirnames, filenames in os.walk(self.getBlobDir()):
            for filename in filenames:
                filename = os.path.join(dirpath, filename)
                if filename not in known_files:
                    orphans.append(filename)
        return orphans

    security.declarePrivate('getFilenameForContent')
    def getFilenameForContent(self, content):
        """ Return the filename for an object """
        blob_dir = content._blob_dir
        field = content.getPrimaryField()
        if not isinstance(field, BlobFileField):
            # not a blob file
            return
        blob = field.getRaw(content).data
        return filename_from_blob(blob, blob_dir)
